@@ -52,6 +52,7 @@
 #   set.edge.value
 #   set.network.attribute
 #   set.vertex.attribute
+#   valid.eids
 #
 ######################################################################
 
@@ -95,7 +96,7 @@ add.edges<-function(x, tail, head, names.eval=NULL, vals.eval=NULL, ...){
   else if(!is.list(vals.eval))
     vals.eval<-as.list(rep(vals.eval,length=length(names.eval)))
   if(length(unique(c(length(tail),length(head),length(names.eval), length(vals.eval))))>1)
-    stop("head, tail, and value lists passed to add.edges must be of the same length!\n")
+    stop("head, tail, names.eval and vals.eval lists passed to add.edges must be of the same length!\n")
   edge.check<-list(...)$edge.check
   if(is.null(edge.check))
     edge.check<-FALSE
@@ -488,9 +489,11 @@ is.bipartite<-function(x){
   else
     bip <- get.network.attribute(x,"bipartite")
   if(is.null(bip)){
-   FALSE
+   return(FALSE)
+  } else if (is.logical(bip)){
+   return(bip)  
   }else{
-   bip>0
+   return(bip>=0)
   }
 }
 
@@ -692,7 +695,7 @@ permute.vertexIDs<-function(x,vids){
     stop("Invalid permutation vector in permute.vertexIDs.")
   if(is.bipartite(x)){  #If bipartite, enforce partitioning
     bpc<-get.network.attribute(x,"bipartite")
-    if(any(vids[1:bpc]>bpc)||(vids[(bpc+1):n]<=bpc))
+    if(any(vids[0:bpc]>bpc)||(vids[(bpc+1):n]<=bpc))
       warning("Performing a cross-mode permutation in permute.vertexIDs.  I hope you know what you're doing....")
   }
   #Return the permuted graph
@@ -824,3 +827,13 @@ set.vertex.attribute<-function(x,attrname,value,v=seq_len(network.size(x))){
   invisible(x)
 }
 
+# valid.eids  returns a list of non-null edge ids for a given network
+valid.eids <-function(x){
+  # maybe should omit class test for speed?
+  if (!is.network(x)){
+    stop("cannot determine non-null edge ids because argument x is not a network object")
+  }
+  # get the ids of all the non-null elements on the edgelist of x
+  return(which(!sapply(x$mel,is.null)))
+}
+
